library(quanteda)
library(quanteda.textstats)
library(readtext)
library(ggplot2)

corpus <- readtext("./*.txt",
                   docvarsfrom = "filename",
                   docvarnames = c("gutenberg_id", "author"),
                   dvsep = "_") |> corpus()
summary(corpus)
head(corpus[1,])
?tokens
tokens <- tokens(corpus, remove_punct = T) |> tokens_tolower()
tokens[1,]
types <- types(tokens)
head(types)
length(types)

dfm <- dfm(tokens)
head(dfm)
head(dfm[3,])
topfeatures(dfm[3,])
tokens[3,]
stats <- topfeatures(dfm, n=nfeat(dfm))
head(stats)
stats <- data.frame(feat=names(stats), freq=stats, rank=1:length(stats))
ggplot(stats, aes(x=rank,y=freq))+
  geom_point()+
  theme_classic()+ 
  coord_trans(x="log2", y="log2")

stopwords("en")
tokens_nostop <- tokens |> tokens_remove(stopwords("en"))
dfm_nostop <- dfm(tokens_nostop)
dfm_nostop
topfeatures(dfm_nostop)
# types_nostop <- types(tokens_nostop)
# "s" %in% types_nostop
#

dfm_nostop
dfm_prop <- dfm_weight(dfm_nostop, scheme  = "prop")
dfm_prop
head(dfm_prop[3,])
topfeatures(dfm_prop[3,])
#

dfm_tfidf <- dfm_tfidf(dfm_nostop)
dfm_tfidf
dfm_tfidf[1,]
topfeatures(dfm_tfidf[1,])
topfeatures(dfm_tfidf[3,])
topfeatures(dfm_tfidf[5,], n = 50)
#

cosine <- function(A,B) { (sum(A*B))/sqrt((sum(A^2))*(sum(B^2))) }

ae <- as.vector(dfm_tfidf[3,])
ae2 <- as.vector(dfm_tfidf[5,])
nt <- as.vector(dfm_tfidf[1,])
cosine(ae, nt)
cosine(ae2, nt)
cosine(ae, ae2)
#
similarities <- sapply(1:ndoc(dfm_tfidf), function(d){
  cosine(ae, as.vector(dfm_tfidf[d,]) )
})
similarities
docnames(dfm_tfidf)
docnames(dfm_tfidf)[order(similarities, decreasing = T)]

jaccard <- function(a, b) {
  intersection = length(intersect(a, b))
  union = length(a) + length(b) - intersection
  return (intersection/union)
}

ae <- featnames(dfm)[as.vector(dfm[3,] > 0)]
nt <- featnames(dfm)[as.vector(dfm[1,] > 0)]
jaccard(ae, nt)
